#import "SpeechSynthesisPatch.h"
#import <AppKit/NSSpeechSynthesizer.h>

@implementation SpeechSynthesisPatch : QCPatch

+ (int)executionMode
{
	return 1;
}
+ (BOOL)allowsSubpatches
{
	return NO;
}
+ (int)timeMode
{
	return 1;
}


- (id)initWithIdentifier:(id)fp8
{
	id z=[super initWithIdentifier:fp8];
	if(z)
	{
		speech = nil;

		// currently, we have continuous, per-trigger, and per-trigger(hold) modes
		[inputCountMode setMaxIndexValue:2];
	}
	return z;
}


- (id)setup:(id)fp8
{
	oldValue = FALSE;
	speech = [[NSSpeechSynthesizer alloc] init];
	return fp8;
}
- (void)cleanup:(id)fp8
{
	[speech release];
}


- (void)enable:(id)fp8
{
}
- (void)disable:(id)fp8
{
	if([speech isSpeaking])
		[speech stopSpeaking];
}

- (BOOL)execute:(id)fp8 time:(double)fp12 arguments:(id)fp20
{
	//NSLog(@"execute: %i %i %i\n",[inputTrigger booleanValue], oldValue, speech);
	if([inputTrigger booleanValue] == TRUE && oldValue == FALSE && [speech isSpeaking] == FALSE)
	{
		// if we're in per-trigger (hold) mode, we need to clean on trigger.  This is safe to do if we're in hold mode as well, so we're
		// actually only going to see if we're in continuous mode
		if([inputCountMode indexValue] != 0)
		{
			spokenWords = 0;
			[outputWords setDoubleValue:0];
		}

		//NSLog(@"SS: setVoice\n");
		if([speech setVoice:[[NSSpeechSynthesizer availableVoices] objectAtIndex:[inputVoice indexValue]]] == YES)
		{
			//NSLog(@"SS: setUsesFeedbackWindow\n");
			[speech setUsesFeedbackWindow:FALSE];
			//NSLog(@"SS: setDelegate\n");
			[speech setDelegate:self];
			//NSLog(@"SS: startSpeakingString\n");
			[speech startSpeakingString:[inputString stringValue]];
			//NSLog(@"SS: finished ss calls\n");
		}
		else	// failed to set voice?
		{
			NSLog(@"SpeechSynthesisPatch: setVoice failure\n");
			oldValue = FALSE;
		}
	}
	
	oldValue = [inputTrigger booleanValue];
	return YES;
}

- (void)speechSynthesizer:(NSSpeechSynthesizer *)sender didFinishSpeaking:(BOOL)success
{
	// clean up
	// if we're in per-trigger (non-hold) mode, we need to clean when we're done talking.
	if([inputCountMode indexValue] == 1)
	{
		spokenWords = 0;
		[outputWords setDoubleValue:0];
	}
}

- (void)speechSynthesizer:(NSSpeechSynthesizer *)sender willSpeakWord:(NSRange)wordToSpeak ofString:(NSString *)text
{
	++spokenWords;
	[outputWords setDoubleValue:(double)spokenWords];
}

- (void)speechSynthesizer:(NSSpeechSynthesizer *)sender willSpeakPhoneme:(short)phonemeOpcode
{
	[outputPhoneme setDoubleValue:(double)phonemeOpcode];
}

@end
